package com.DeviceTest;

import android.Manifest;
import android.app.Activity;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import androidx.core.app.ActivityCompat;
import androidx.core.content.ContextCompat;

import com.DeviceTest.serialPort.SerialPort;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.security.InvalidParameterException;
import java.util.Timer;
import java.util.TimerTask;

/**
 * Created by Administrator on 2018/1/10 0010.
 * 串口测试
 */
public class SerialActivity extends Activity {

    private InputStream mInputStream;
    protected OutputStream mOutputStream;
    private ReadThread mReadThread;
    private SendingThread mSendingThread;
    private SerialPort mSerialPort = null;

    private final String TEST_SEND = "deviceTest";
    private final String RK3288 = "rk3288";
    private final String RK3368 = "rk3368";
    private final String RK3566 = "rk3566";
    private final String RK3568 = "rk3568";
    private final String RK3399 = "rk3399";
    private static String TAG = "SerialPort";
    private final int UPDATE_UI = 105;
    private static boolean isConnectionSuccess = true;
    private TextView textView;
    private Button btn_back;
    private Button btn_restart;
    private Timer timer;


    private Handler handler = new Handler() {
        @Override
        public void dispatchMessage(Message msg) {
            switch (msg.what) {
                case UPDATE_UI:
                    if (!isConnectionSuccess) {
                        Log.d(TAG, "fail:" + isConnectionSuccess);
                        textView.setText(getString(R.string.fail));
                    } else {
                        Log.d(TAG, "success:" + isConnectionSuccess);
                        textView.setText(getString(R.string.success));
                        timer.cancel();
                    }
                    break;
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_serial);

        if (ContextCompat.checkSelfPermission(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)
                != PackageManager.PERMISSION_GRANTED) {
            ActivityCompat.requestPermissions(this,
                    new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE,
                            Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.MOUNT_UNMOUNT_FILESYSTEMS}, 0);
        }

        btn_back = findViewById(R.id.btn_back);
        btn_restart = findViewById(R.id.btn_restart);
        textView = findViewById(R.id.tv_status);
        mSerialPort = new SerialPort();
        btn_back.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                finish();
            }
        });
        btn_restart.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                startTest();
            }
        });

//        try {
            getSerialPort();
//        } catch (IOException e) {
//            e.printStackTrace();
//            textView.setText(getString(R.string.fail));
//            return;
//        } catch (SecurityException e) {
//            e.printStackTrace();
//            textView.setText(getString(R.string.serial_connect_fail));
//            return;
//        }

        startTest();

    }

    private void startTest() {

        if (mSerialPort != null) {
            textView.setText(getString(R.string.connecting));
            mOutputStream = mSerialPort.getOutputStream();
            mInputStream = mSerialPort.getInputStream();
            if (mOutputStream!= null || mInputStream != null){
                textView.setText(getString(R.string.success));
            }else {
                textView.setText(getString(R.string.fail));
            }
            mReadThread = new ReadThread();
            mReadThread.start();
            mSendingThread = new SendingThread();
            mSendingThread.start();
            timer = new Timer();
            timer.schedule(new TimerTask() {
                @Override
                public void run() {
                    handler.sendEmptyMessage(UPDATE_UI);
                }
            }, 5000);
        }
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        closeSerialPort();
        if (null != mReadThread) {
            mReadThread.interrupt();
            mReadThread = null;
        }
        if (null != mSendingThread) {
            mSendingThread.interrupt();
            mSendingThread = null;
        }
    }


//    public static String getCpu() {
//        return SystemProperties.get("ro.board.platform");
//    }

    public void getSerialPort(){
//        if (mSerialPort == null) {
            String cpu = getCpu().toLowerCase();
            Log.e("SerialPort","cpu = "+cpu);
//            String serialPortPath = "/dev/ttyS3";
//            if (RK3288.equalsIgnoreCase(cpu)) {
//                serialPortPath = "/dev/ttyS3";
//            } else if (RK3368.equalsIgnoreCase(cpu)) {
//                serialPortPath = "/dev/ttyS1";
//            }else if (RK3566.equalsIgnoreCase(cpu)){
//                serialPortPath = "/dev/ttyS7";
//            } else if (RK3568.equalsIgnoreCase(cpu)){
//                serialPortPath = "/dev/ttyS3";
//            } else if (RK3399.equalsIgnoreCase(cpu)){
//                serialPortPath = "/dev/ttyS4";
//            }
            String devicePath = "/dev/ttyS4";
            if (cpu.equalsIgnoreCase("rk3288")) {
                if (getProjectFlag().contains("W23")) { //W23/W23M/W23D
                    devicePath ="/dev/ttyS4";
                } else {
                    devicePath ="/dev/ttyS3";
                }
            } else if (cpu.equalsIgnoreCase("rk3368")) {
                devicePath = "/dev/ttyS1";
            } else if (cpu.equalsIgnoreCase("rk3566")) { //1826X
                devicePath = "/dev/ttyS7";
            } else if (cpu.equalsIgnoreCase("rk3568")) {
                if (getProjectFlag().contains("W23")) { //W23 Plus/W23M Plus/W23D Plus
                    devicePath = "/dev/ttyS4";
                } else {
                    devicePath = "/dev/ttyS3";
                }
            } else if (cpu.equalsIgnoreCase("rk3399")) {
                devicePath = "/dev/ttyS4";
            }

            Log.e("SerialPort","serialPortPath = "+devicePath);
            //mSerialPort = new SerialPort(new File(devicePath), 9600, 0);
            boolean b = mSerialPort.serialOpen(new File(devicePath), 9600, 0);
        Log.e("SerialPort","b = "+b);
            textView.setText(b == true?getString(R.string.success):getString(R.string.fail));

//        }
//        return mSerialPort;
    }

    public void closeSerialPort() {
        if (mSerialPort != null) {
            mSerialPort.close();
            mSerialPort = null;
        }
    }

    private class ReadThread extends Thread {
        @Override
        public void run() {
            super.run();
            while (!isInterrupted()) {
                try {
                    byte[] buffer = new byte[TEST_SEND.length()];
                    if (mInputStream == null) {
                        handler.sendEmptyMessage(UPDATE_UI);
                        return;
                    }
                    mInputStream.read(buffer);
                    if (buffer.length > 0 && buffer[0] == TEST_SEND.getBytes()[0]) {
                        isConnectionSuccess = true;
                        Log.d(TAG, "read data: " + buffer[0] + " == " + buffer[1] + " == " + buffer[2] + " == " + buffer[3] + " == " + buffer[4] + " == " + buffer[5] + " == " + buffer[6]);
//                        handler.sendEmptyMessage(UPDATE_UI);
                        Log.e("SerialPort","mInputStream = "+(mInputStream != null));
                    }
                } catch (IOException e) {
                    e.printStackTrace();
                    handler.sendEmptyMessage(UPDATE_UI);
                }
            }
        }
    }

    private class SendingThread extends Thread {
        @Override
        public void run() {
            try {
//                isConnectionSuccess = false;
                if (mOutputStream != null) {
                    byte[] mBuffer = TEST_SEND.getBytes();
                    Log.d(TAG, "send data: " + mBuffer[0] + " == " + mBuffer[1] + " == " + mBuffer[2] + " == " + mBuffer[3] + " == " + mBuffer[4] + " == " + mBuffer[5] + " == " + mBuffer[6]);
                    mOutputStream.write(mBuffer);
                    Log.e("SerialPort","mOutputStream = "+(mOutputStream != null));
                }
//                else {
//                    handler.sendEmptyMessage(UPDATE_UI);
//                }
            } catch (IOException e) {
                e.printStackTrace();
                handler.sendEmptyMessage(UPDATE_UI);
            }
        }
    }

    public static String getSystemProperties(String property,String defaultStr) {
        String value = defaultStr;
        try {
            Class clazz = Class.forName("android.os.SystemProperties");
            Method getter = clazz.getDeclaredMethod("get", String.class);
            value = (String) getter.invoke(null, property);
        } catch (Exception e) {
            Log.d("TAG", "Unable to read system properties");
        }
        return value;
    }

    public static String getCpu() {
        String cpu = getSystemProperties("ro.board.platform.cpu","");
        if(cpu.length()>1)
            return cpu;
        else{
            return getSystemProperties("ro.board.platform","");
        }
    }

    public static String getProjectFlag() {
        String projectflag = "";
        String flags = getFlagBit();
        if (TextUtils.isEmpty(flags) || flags.length() < 30) {
            return projectflag;
        }
        projectflag = flags.substring(0, 5);
        return projectflag;
    }

    public static String getFlagBit() {
        return getSystemProp("sys.neostra_oem_id");
    }

    public static String getSystemProp(String cmd) {
        return getSystemProperties(cmd,"");
    }

}
